use crate::constants::CLOCK_PERIOD;
use specs::prelude::*;
use std::time::{Duration, Instant};

use crate::resources::{Clock, StopClock};

pub struct TimeSystem {}

#[allow(clippy::type_complexity)]
impl<'a> System<'a> for TimeSystem {
    type SystemData = (WriteExpect<'a, Clock>, ReadExpect<'a, StopClock>);

    fn run(&mut self, (mut clock, stop_clock): Self::SystemData) {
        if !stop_clock.0 {
            Self::try_tick(&mut clock);
        } else {
            Self::reset(&mut clock);
        }
    }
}

impl TimeSystem {
    pub fn try_tick(clock: &mut Clock) {
        if Instant::now() - clock.last_ticked > Duration::from_secs_f32(CLOCK_PERIOD) {
            Self::tick(clock);
        } else {
            clock.has_ticked = false;
        }
    }

    pub fn force_tick(clock: &mut Clock) {
        Self::tick(clock);
    }

    fn tick(clock: &mut Clock) {
        clock.has_ticked = true;
        clock.last_ticked = Instant::now();
        clock.ticks += 1;
    }

    pub fn reset(clock: &mut Clock) {
        clock.last_ticked = Instant::now();
        clock.has_ticked = false;
    }
}
